# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['globus_action_provider_tools']

package_data = \
{'': ['*']}

install_requires = \
['dogpile.cache>=0.7.1,<0.8.0',
 'globus-nexus-client>=0.2.8,<0.3.0',
 'globus-sdk>=1.7.1,<2.0.0',
 'openapi-core>=0.11.0,<0.12.0']

setup_kwargs = {
    'name': 'globus-action-provider-tools',
    'version': '0.6',
    'description': 'Tools to help developers build services that implement the Action Provider specification.',
    'long_description': "Action Provider Tools\n=====================\n\nThis is an experimental and unsupported toolkit to help developers build Action Providers for use in Globus Automate Flows.\n\nAs this is experimental, no support is implied or provided for any sort of use of this package. It is published for ease of distribution among those planning to use it for its intended, experimental, purpose.\n\n\nToolkit Components\n------------------\n\nThis toolkit provides two main components:\n\n1. Authentication helpers that make it easier to validate Globus Auth tokens and determine if a given request should be authorized\n\n2. An `OpenAPI v3 specification <http://spec.openapis.org/oas/v3.0.2>`_ and associated helpers that can be used to validate incoming requests and verify the responses your Action Provider generates. This document also defines the interface which must be supported by your REST API to have it function as an Action Provider.\n\n\nInstallation\n------------\n\nInstallation is via PyPi using, for example: ``pip install globus-action-provider-tools``.\n\n\nUsage\n-----\n\n\nAuthentication\n---------------\n\nThe authentication helpers can be used in your action provider as follows:\n\n.. code-block:: python\n\n    from globus_action_provider_tools.authentication import TokenChecker\n    # You will need to register a client and scope(s) in Globus Auth\n    # Then initialize a TokenChecker instance for your provider:\n    checker = TokenChecker(\n        client_id='YOUR_CLIENT_ID',\n        client_secret='YOUR_CLIENT_SECRET',\n        expected_scopes=['https://auth.globus.org/scopes/YOUR_SCOPES_HERE'],\n        expected_audience='YOUR_CLIENT_NAME',\n    )\n\n\n(expected_audience should be unnecessary in the near future)\n\nWhen a request comes in, use your TokenChecker to validate the access token from the HTTP Authorization header.\n\n.. code-block:: python\n\n    access_token = request.headers['Authorization'].replace('Bearer ', '')\n    auth_state = checker.check_token(access_token)\n\n\nThe AuthState has several properties and methods that will make it easier for you to decide whether or not to allow a request to proceed:\n\n.. code-block:: python\n\n    # This user's Globus identities:\n    auth_state.identities\n    # frozenset({'urn:globus:auth:identity:9d437146-f150-42c2-be88-9d625d9e7cf9',\n    #           'urn:globus:auth:identity:c38f015b-8ad9-4004-9160-754b309b5b33',\n    #           'urn:globus:auth:identity:ffb5652b-d418-4849-9b57-556656706970'})\n    # Groups this user is a member of:\n    auth_state.groups\n    # frozenset({'urn:globus:groups:id:606dbaa9-3d57-44b8-a33e-422a9de0c712',\n    #           'urn:globus:groups:id:d2ff42bc-c708-460f-9e9b-b535c3776bdd'})\n\n\nYou'll notice that both groups and identities are represented as strings that unambiguously signal what type of entity they represent. This makes it easy to merge the two sets without conflict, for situations where you'd like to work with a single set containing all authentications:\n\n.. code-block:: python\n\n    all_principals = auth_state.identities.union(auth_state.groups)\n\n\nThe AuthState object also offers a helper method, `check_authorization()` that is designed to help you test whether a request should be authorized:\n\n.. code-block:: python\n\n    resource_allows = ['urn:globus:auth:identity:c38f015b-8ad9-4004-9160-754b309b5b33']\n    auth_state.check_authorization(resource_allows)\n    # True\n\n\nThis method also accepts two special string values, ``'public'`` and ``'all_authenticated_users'``, together with keyword arguments that enable their use:\n\n.. code-block:: python\n\n    resource_allows = ['public']\n    auth_state.check_authorization(resource_allows, allow_public=True)\n    # True\n    resource_allows = ['all_authenticated_users']\n    auth_state.check_authorization(resource_allows, allow_all_authenticated_users=True)\n   # True\n\n\nCaching\n-------\n\nTo avoid excessively taxing Globus Auth, the ``AuthState`` will, by default, cache identities and group memberships for 30 seconds.\n\nThe cache is initialized when you first instantiate your ``TokenChecker()``.  You should only need to create one TokenChecker instance for your application, and then you can re-use it to check each new token. If you do try to make multiple instances, you may get an exception:\n\n> ``dogpile.cache.exception.RegionAlreadyConfigured: This region is already configured``\n\nbecause it's trying to re-initialize a cache that's already been set up.\n\n\nValidation\n----------\n\nThere is an OpenAPI v3 specification for the Action Provider API available as described above. You can use any tools that accept OpenAPI v3 to validate requests to and responses from your service, but this toolkit offers some pre-configured validators using the openapi-core library.\n\n.. code-block:: python\n\n    from globus_action_provider_tools.validation import request_validator, response_validator\n    # Validating a request\n    result = request_validator.validate(request)\n    # Or a response:\n    result = response_validator.validate(request, response)\n    # raise errors if invalid\n    result.raise_for_errors()\n    # or get list of errors\n    errors = result.errors\n\n\nNote that the ``request`` and ``response`` objects passed to the validators must conform to the `BaseOpenAPI <https://github.com/p1c2u/openapi-core/blob/master/openapi_core/wrappers/base.py>`_ request and response interfaces. If you're using Flask, you can use the `provided wrappers <https://github.com/p1c2u/openapi-core/blob/master/openapi_core/wrappers/flask.py#L10>`_, but if you're using a different web framework, you'll need to write your own small wrapper class.\n\n",
    'author': 'Jim Pruyne',
    'author_email': 'pruyne@globus.org',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
